from sdetools.alm_integration.alm_plugin_base import AlmTask, STATUS
from sdetools.extlib.iso8601.iso8601 import parse


class JIRATask(AlmTask):
    """ Representation of a task in JIRA """

    def __init__(self, task_id, alm_id, priority, status, resolution,
                 timestamp, done_statuses, versions):
        self.task_id = task_id
        self.alm_id = alm_id
        self.priority = priority
        self.status = status
        self.resolution = resolution
        self.timestamp = timestamp
        self.done_statuses = done_statuses  # comma-separated list
        self.versions = versions # array of version names

    @staticmethod
    def sanitize_label(label):
        return label.replace(' ','-')

    def get_task_id(self):
        return self.task_id

    def get_alm_id(self):
        return self.alm_id

    def get_status(self):
        """ Translate Jira status to SDE status - disregard case """
        if self.status.lower() in [status.lower() for status in self.done_statuses]:
            return STATUS.DONE
        else:
            return STATUS.TODO

    def get_timestamp(self):
        """ Returns a datetime object """
        return parse(self.timestamp)
