# coding=utf-8
import json
import os
import unittest

from sdetools.sdelib.restclient import RESTBase
from sdetools.sdelib.commons import get_directory_of_current_module
from sdetools.sdelib.mod_mgr import ReturnChannel, load_modules
from sdetools.sdelib.conf_mgr import Config

CONF_FILE_LOCATION = 'test_settings.conf'


def stdout_callback(obj):
    print obj


class TestRestClient(unittest.TestCase):

     def setUp(self):
        self.ret_chn = ReturnChannel(stdout_callback, {})
        command_list = load_modules()
        self.conf_path = os.path.join(get_directory_of_current_module(self), CONF_FILE_LOCATION)
        self.ret_chn = ReturnChannel(stdout_callback, {})
        self.config = Config('help', '', command_list, [], self.ret_chn, 'shell')
        self.rest_client = RESTBase('sde', 'conf_name', self.config)
        self.config.import_custom_options()
        Config.parse_config_file(self.config, self.conf_path)

     def test_context_root_is_empty(self):
        self._check_context_root_matches_target('', '')

     def test_context_root_is_slash(self):
        self._check_context_root_matches_target('/', '')

     def test_context_root_is_notempty(self):
        self._check_context_root_matches_target('branch', '/branch')

     def _check_context_root_matches_target(self, context_root, target):
        self.rest_client.config['sde_context_root'] = context_root
        self.assertTrue(self.rest_client._get_context_root() == target, 'Expected an empty context root')

class TestRestClientUnicode(TestRestClient):
    """
    Test our methods for handling Unicode in the API
    """
    def setUp(self):
        super(TestRestClientUnicode, self).setUp()

        self.unicode_string = u'unicode string 有'
        self.encoded_string = 'unicode string \xe6\x9c\x89'
        self.unicode_dict = {u'unicode key 有': u'unicode value 有'}
        self.encoded_dict = {'unicode key \xe6\x9c\x89': 'unicode value \xe6\x9c\x89'}
        self.unicode_list = [u'unicode value 有']
        self.encoded_list = ['unicode value \xe6\x9c\x89']


    def test_utf8_list(self):
        expected = self.encoded_list
        actual = self.rest_client.encode_list_utf8(self.unicode_list)

        self.assertEqual(expected, actual)

    def test_utf8_dict(self):
        expected = self.encoded_dict
        actual = self.rest_client.encode_dict_utf8(self.unicode_dict)

        self.assertEqual(expected, actual)

    def test_utf8_unicode(self):
        expected = self.encoded_string
        actual = self.rest_client.encode_string_utf8(self.unicode_string)

        self.assertEqual(expected, actual)

        # Ensure idempotence
        expected = self.encoded_string
        actual = self.rest_client.encode_string_utf8(self.encoded_string)

        self.assertEqual(expected, actual)

    def test_encode_post_args(self):
        expected = json.dumps(self.encoded_string)
        actual = self.rest_client.encode_post_args(self.unicode_string)

        self.assertEqual(expected, actual)

        expected = json.dumps(self.encoded_list)
        actual = self.rest_client.encode_post_args(self.unicode_list)

        self.assertEqual(expected, actual)

        expected = json.dumps(self.encoded_dict)
        actual = self.rest_client.encode_post_args(self.unicode_dict)

        self.assertEqual(expected, actual)

    def test_urlencode_dict(self):
        expected = 'unicode+key+%E6%9C%89=unicode+value+%E6%9C%89'
        actual = self.rest_client.urlencode(self.unicode_dict)

        self.assertEqual(expected, actual)

    def test_urlencode_string(self):
        expected = self.encoded_string
        expected = 'unicode+string+%E6%9C%89'
        actual = self.rest_client.urlencode(self.unicode_string)

        self.assertEqual(expected, actual)
